require_relative '../request_master'
require_relative 'models'

module Kernel

  def Numbers (array)
    Numbers.new(array)
  end

end

class Numbers

  def initialize (array)
    @array = array
  end

  def method_missing (method, *args, &block)
    @array.send(method, *args, &block)
  end

  def sum
    @array.inject(0) { |acc, num| acc + num }
  end

  def average
    sum.to_f / @array.size
  end

end

module RequestMaster

  class ReqMaster

    def dom_id (url)
      Website[domain_url: url].id
    end

    def ip_stats (ip, dom_id)
      requests = BrowserRequest.where(domain: dom_id, ip_address: ip)
      num = requests.count

      if num >= 2
        time_span = (requests.last.time.to_date - requests.first.time.to_date + 1).to_f
        avg = (num / time_span).to_f.round(1)
      elsif num == 1
        avg = num
        time_span = 1.0
      else
        avg = nil
        time_span = 0.0
      end

      OpenStruct.new(num: num, avg: avg, days: time_span.round(1))
    end

    def all_ip_stats (dom_id)

      all_ips = IpAddress.select(:public_ip).to_a
      all_ips.map!(&:public_ip)

      all_avg = Numbers([])
      all_total = Numbers([])

      output = all_ips.inject([]) { |acc, ip|
        stats = ip_stats(ip, dom_id)

        if stats.num >= 50
          all_total << stats.num
          all_avg << stats.avg
          acc << {ip => stats.avg}
        else
          acc
        end
      }

      total_stats = {
          min_total: all_total.min,
          max_total: all_total.max,
          avg_total: all_total.average.round,
          min_avg: all_avg.min,
          max_avg: all_avg.max,
          avg_avg: all_avg.average.round
      }

      output << total_stats
    end

  end

end
